@@ -15,7 +15,7 @@ from scipy.stats import chi2
 from scipy.ndimage.filters import gaussian_filter1d
 
 from .utils import Epoch
-from .utils import printProgressBar, get_spike_depths
+from .utils import printProgressBar, get_spike_positions
 
 
 def calculate_metrics(spike_times, spike_clusters, amplitudes, pc_features, pc_feature_ind, params,
@@ -125,7 +125,7 @@ def calculate_metrics(spike_times, spike_clusters, amplitudes, pc_features, pc_f
         metrics = pd.concat((metrics, pd.DataFrame(data=OrderedDict((('cluster_id', cluster_ids_out),
                                                                      ('firing_rate', firing_rate),
                                                                      ('presence_ratio', presence_ratio),
-                                                                     ('isi_viol', isi_viol),
+                                                                     ('isi_violation', isi_viol),
                                                                      ('amplitude_cutoff', amplitude_cutoff),
                                                                      ('isolation_distance', isolation_distance),
                                                                      ('l_ratio', l_ratio),
@@ -147,8 +147,11 @@ def calculate_metrics(spike_times, spike_clusters, amplitudes, pc_features, pc_f
 
 # ===============================================================
 
-def calculate_isi_violations(spike_times, spike_clusters, total_units, isi_threshold, min_isi, duration, verbose=True):
-    cluster_ids = np.unique(spike_clusters)
+def calculate_isi_violations(spike_times, spike_clusters, total_units, isi_threshold, min_isi, duration, spike_cluster_subset=None,  verbose=True):
+    if spike_cluster_subset is not None:
+        cluster_ids = spike_cluster_subset
+    else: 
+        cluster_ids = np.unique(spike_clusters)
 
     viol_rates = np.zeros((total_units,))
 
@@ -166,8 +169,11 @@ def calculate_isi_violations(spike_times, spike_clusters, total_units, isi_thres
     return viol_rates
 
 
-def calculate_presence_ratio(spike_times, spike_clusters, total_units, duration, verbose=True):
-    cluster_ids = np.unique(spike_clusters)
+def calculate_presence_ratio(spike_times, spike_clusters, total_units, duration, spike_cluster_subset=None, verbose=True):
+    if spike_cluster_subset is not None:
+        cluster_ids = spike_cluster_subset
+    else: 
+        cluster_ids = np.unique(spike_clusters)
 
     ratios = np.zeros((total_units,))
 
@@ -184,9 +190,12 @@ def calculate_presence_ratio(spike_times, spike_clusters, total_units, duration,
 
 
 
-def calculate_num_spikes(spike_times, spike_clusters, total_units, verbose=True):
-    cluster_ids = np.unique(spike_clusters)
+def calculate_num_spikes(spike_times, spike_clusters, total_units, spike_cluster_subset=None, verbose=True):
     num_spikes = np.zeros((total_units,))
+    if spike_cluster_subset is not None:
+        cluster_ids = spike_cluster_subset
+    else: 
+        cluster_ids = np.unique(spike_clusters)
 
     for idx, cluster_id in enumerate(cluster_ids):
 
@@ -199,8 +208,12 @@ def calculate_num_spikes(spike_times, spike_clusters, total_units, verbose=True)
     return num_spikes
 
 
-def calculate_firing_rates(spike_times, spike_clusters, total_units, duration, verbose=True):
-    cluster_ids = np.unique(spike_clusters)
+def calculate_firing_rates(spike_times, spike_clusters, total_units, duration, spike_cluster_subset=None, verbose=True):
+    if spike_cluster_subset is not None:
+        cluster_ids = spike_cluster_subset
+    else: 
+        cluster_ids = np.unique(spike_clusters)
+    
     firing_rates = np.zeros((total_units,))
 
     for idx, cluster_id in enumerate(cluster_ids):
@@ -215,8 +228,11 @@ def calculate_firing_rates(spike_times, spike_clusters, total_units, duration, v
     return firing_rates
 
 
-def calculate_amplitude_cutoff(spike_clusters, amplitudes, total_units, verbose=True):
-    cluster_ids = np.unique(spike_clusters)
+def calculate_amplitude_cutoff(spike_clusters, amplitudes, total_units, spike_cluster_subset=None,  verbose=True):
+    if spike_cluster_subset is not None:
+        cluster_ids = spike_cluster_subset
+    else: 
+        cluster_ids = np.unique(spike_clusters)
 
     amplitude_cutoffs = np.zeros((total_units,))
 
@@ -241,14 +257,20 @@ def calculate_pc_metrics(spike_clusters,
                          n_neighbors,
                          min_num_pcs=10,
                          metric_names=None,
-                         seed=None, verbose=True):
+                         seed=None, 
+                         spike_cluster_subset=None,  
+                         verbose=True):
     assert (num_channels_to_compare % 2 == 1)
     half_spread = int((num_channels_to_compare - 1) / 2)
 
     if metric_names is None:
         metric_names = ['isolation_distance', 'l_ratio', 'd_prime', 'nearest_neighbor']
 
-    cluster_ids = np.unique(spike_clusters)
+    all_cluster_ids = np.unique(spike_clusters)
+    if spike_cluster_subset is not None:
+        cluster_ids = spike_cluster_subset
+    else: 
+        cluster_ids = all_cluster_ids
 
     peak_channels = np.zeros((total_units,), dtype='uint16')
     isolation_distances = np.zeros((total_units,))
@@ -257,7 +279,7 @@ def calculate_pc_metrics(spike_clusters,
     nn_hit_rates = np.zeros((total_units,))
     nn_miss_rates = np.zeros((total_units,))
 
-    for idx, cluster_id in enumerate(cluster_ids):
+    for idx, cluster_id in enumerate(all_cluster_ids):
         for_unit = np.squeeze(spike_clusters == cluster_id)
         pc_max = np.argmax(np.mean(pc_features[for_unit, 0, :], 0))
         peak_channels[cluster_id] = pc_feature_ind[cluster_id, pc_max]
@@ -360,7 +382,9 @@ def calculate_silhouette_score(spike_clusters,
                                pc_features,
                                pc_feature_ind,
                                spikes_for_silhouette,
-                               seed=None, verbose=True):
+                               seed=None, 
+                               spike_cluster_subset=None, 
+                               verbose=True):
     random_spike_inds = np.random.RandomState(seed=seed).permutation(spike_clusters.size)
     random_spike_inds = random_spike_inds[:spikes_for_silhouette]
     num_pc_features = pc_features.shape[1]
@@ -378,25 +402,29 @@ def calculate_silhouette_score(spike_clusters,
 
     cluster_labels = spike_clusters[random_spike_inds]
 
-    cluster_ids = np.unique(cluster_labels)
+    all_cluster_ids = np.unique(spike_clusters)
+    if spike_cluster_subset is not None:
+        cluster_ids = spike_cluster_subset
+    else: 
+        cluster_ids = all_cluster_ids
 
     SS = np.empty((total_units, total_units))
     SS[:] = np.nan
 
+    seen_unit_pairs = set()
     for idx1, i in enumerate(cluster_ids):
 
         if verbose:
             printProgressBar(idx1 + 1, len(cluster_ids))
 
-        for idx2, j in enumerate(cluster_ids):
-
-            if j > i:
+        for idx2, j in enumerate(all_cluster_ids):
+            if (i,j) not in seen_unit_pairs and (j,i) not in seen_unit_pairs and i != j:
                 inds = np.in1d(cluster_labels, np.array([i, j]))
                 X = all_pcs[inds, :]
                 labels = cluster_labels[inds]
-
                 if len(labels) > 2:
-                    SS[i, j] = silhouette_score(X, labels)
+                    SS[i, j] = silhouette_score(X, labels, random_state=seed)
+                seen_unit_pairs.add((i,j))
 
     with warnings.catch_warnings():
       warnings.simplefilter("ignore")
@@ -414,16 +442,22 @@ def calculate_drift_metrics(spike_times,
                             interval_length,
                             min_spikes_per_interval,
                             vertical_channel_spacing=10,
+                            channel_locations=None,
+                            spike_cluster_subset=None,
                             verbose=True):
 
     max_drift = np.zeros((total_units,))
     cumulative_drift = np.zeros((total_units,))
 
-    depths = get_spike_depths(spike_clusters, pc_features, pc_feature_ind, vertical_channel_spacing)
+    positions = get_spike_positions(spike_clusters, pc_features, pc_feature_ind, channel_locations,
+                                    vertical_channel_spacing)
     interval_starts = np.arange(np.min(spike_times), np.max(spike_times), interval_length)
     interval_ends = interval_starts + interval_length
 
-    cluster_ids = np.unique(spike_clusters)
+    if spike_cluster_subset is not None:
+        cluster_ids = spike_cluster_subset
+    else: 
+        cluster_ids = np.unique(spike_clusters)
 
     for idx, cluster_id in enumerate(cluster_ids):
 
@@ -432,23 +466,40 @@ def calculate_drift_metrics(spike_times,
 
         in_cluster = spike_clusters == cluster_id
         times_for_cluster = spike_times[in_cluster]
-        depths_for_cluster = depths[in_cluster]
+        positions_for_cluster = positions[in_cluster]
 
-        median_depths = []
+        median_positions = []
 
         for t1, t2 in zip(interval_starts, interval_ends):
-
             in_range = (times_for_cluster > t1) * (times_for_cluster < t2)
 
             if np.sum(in_range) >= min_spikes_per_interval:
-                median_depths.append(np.median(depths_for_cluster[in_range]))
+                median_positions.append(np.median(positions_for_cluster[in_range], 0))
             else:
-                median_depths.append(np.nan)
+                median_positions.append([np.nan, np.nan])
 
-        median_depths = np.array(median_depths)
+        median_positions = np.array(median_positions)
 
-        max_drift[cluster_id] = np.around(np.nanmax(median_depths) - np.nanmin(median_depths), 2)
-        cumulative_drift[cluster_id] = np.around(np.nansum(np.abs(np.diff(median_depths))), 2)
+        # Extract emi-matrix of shifts in positions (used to extract max_drift and cum_drift)
+        position_diffs = np.zeros((len(median_positions), len(median_positions)))
+        for i, pos_i in enumerate(median_positions):
+            for j, pos_j in enumerate(median_positions):
+                if j > i:
+                    if not np.isnan(pos_i[0]) and not np.isnan(pos_j[0]):
+                        position_diffs[i, j] = np.linalg.norm(pos_i - pos_j)
+                    else:
+                        position_diffs[i, j] = 0
+
+        # Maximum drift among all periods
+        if np.any(position_diffs > 0):
+            max_drift[cluster_id] = np.around(np.max(position_diffs[position_diffs > 0]), 2)
+            # The +1 diagonal contains the step-by-step drifts between intervals.
+            # Summing them up we obtain cumulative drift
+            cumulative_drift[cluster_id] = np.around(np.sum(np.diag(position_diffs, 1)), 2)
+        else:
+            # not enough spikes
+            max_drift[cluster_id] = 0
+            cumulative_drift[cluster_id] = 0
 
     return max_drift, cumulative_drift
 
@@ -460,7 +511,7 @@ def calculate_drift_metrics(spike_times,
 # ==========================================================
 
 
-def isi_violations(spike_train, duration, isi_threshold, min_isi=None):
+def isi_violations(spike_train, duration, isi_threshold, min_isi=0):
     """Calculate Inter-Spike Interval (ISI) violations for a spike train.
 
     Based on metric described in Hill et al. (2011) J Neurosci 31: 8699-8705
@@ -474,10 +525,9 @@ def isi_violations(spike_train, duration, isi_threshold, min_isi=None):
     duration : length of recording (seconds)
     isi_threshold : threshold for classifying adjacent spikes as an ISI violation
       - this is the biophysical refractory period
-    min_isi : minimum possible inter-spike interval (default = None)
+    min_isi : minimum possible inter-spike interval (default = 0)
       - this is the artificial refractory period enforced by the data acquisition system
         or post-processing algorithms
-      - if set to None, all spikes will be included
 
     Outputs:
     --------
@@ -488,11 +538,9 @@ def isi_violations(spike_train, duration, isi_threshold, min_isi=None):
     """
     isis_initial = np.diff(spike_train)
 
-    if min_isi is not None:
+    if min_isi > 0:
         duplicate_spikes = np.where(isis_initial <= min_isi)[0]
         spike_train = np.delete(spike_train, duplicate_spikes + 1)
-    else:
-        min_isi = 0
 
     isis = np.diff(spike_train)
     num_spikes = len(spike_train)
@@ -632,7 +680,7 @@ def mahalanobis_metrics(all_pcs, all_labels, this_unit_id):
 
     if n >= 2:
         dof = pcs_for_this_unit.shape[1]  # number of features
-        l_ratio = np.sum(1 - chi2.cdf(pow(mahalanobis_other, 2), dof)) / mahalanobis_other.shape[0]
+        l_ratio = np.sum(1 - chi2.cdf(pow(mahalanobis_other, 2), dof)) / mahalanobis_self.shape[0]
         isolation_distance = pow(mahalanobis_other[n - 1], 2)
         # if math.isnan(l_ratio):
         #     print("NaN detected", mahalanobis_other, VI)
