@@ -1,7 +1,7 @@
 import numpy as np
 import pytest
 
-from spikemetrics.utils import get_spike_depths
+from spikemetrics.utils import get_spike_positions
 from spikemetrics.tests.utils import simulated_pcs_for_one_spike
 
 
@@ -12,6 +12,7 @@ def test_get_spike_depths():
     pc_features, pc_feature_ind = simulated_pcs_for_one_spike(32, peak_chan)
     spike_clusters = np.ones((pc_features.shape[0],), dtype='int') * 0
 
-    depths = get_spike_depths(spike_clusters, pc_features, pc_feature_ind, vertical_channel_spacing=1)
+    positions = get_spike_positions(spike_clusters, pc_features, pc_feature_ind, vertical_channel_spacing=1)
+    depths = positions[:, 1]
 
     assert np.isclose(depths, peak_chan, rtol=0, atol=1e-5)
